#ifndef POLYNOME_HPP
#define POLYNOME_HPP
#include <cmath>
#include <iostream>
#include "complexe.hpp"
using namespace std;

class segment {
public:
  static constexpr double a = 0.0;
  static constexpr double b = 1.0;
};

class segment_ref {
public:
  static constexpr double a = -1.0;
  static constexpr double b = 1.0;
};

template <double (*weight)(double), class segment_type=segment_ref> class polynome {
  int deg;
  complexe * coefs;
public: 
  polynome (){
    deg = 0;
    coefs = NULL;
  }
  polynome (int degree){
    deg = degree;
    coefs = new complexe[degree+1];
    for(int i =0; i<deg+1;i++){
	    coefs[i] = complexe(0,0);
    }
  }
  polynome (const polynome & x){
    deg = x.deg;
    coefs = NULL;
    if(x.coefs != NULL){
      coefs = new complexe[deg+1];
      for(int i =0; i<deg+1;i++){
  	coefs[i] = x.coefs[i];
      }
    }
  }
  ~polynome (){
    if(coefs != NULL){
      delete [] coefs;
    }
  }
  complexe get(int);
  void set(complexe, int);
  polynome primitive();
  polynome derivative();
  complexe integral();
  polynome conjuge();
  polynome operator * (polynome);
  complexe operator , (polynome);
  double norm();
  complexe operator () (complexe);
  complexe operator [] (int i);
  polynome & operator = (const polynome &);
  int testu_1();
  int all_testsu();

private:
  polynome convert(int deg2){
    if( deg2 < deg){
      cout<<"convertion pas possible"<<endl;
    }
    polynome res(deg2);
    for(int i=0;i<deg+1;i++){
	    res.coefs[i] = coefs[i];
    }
    return res;   
  }
  
};		   


template <double (*weight)(double),class segment_type> complexe polynome<weight,segment_type>:: get(int i){
  complexe res;
  res= coefs[i];
  return res;
};

template <double (*weight)(double),class segment_type> void polynome<weight,segment_type>::set(complexe x, int i) {
  coefs[i] = x;
};

template <double (*weight)(double),class segment_type> polynome<weight,segment_type> polynome<weight,segment_type>::conjuge(){ 
  polynome<weight,segment_type> res(deg); 
  for(int i=0;i<deg+1;i++){ 
    res.coefs[i]=coefs[i].conjuge(); 
  } 
  return res;
}

template <double (*weight)(double),class segment_type> complexe polynome<weight,segment_type>::operator [](int i){ 
  return coefs[i];
}

template <double (*weight)(double),class segment_type> complexe polynome<weight,segment_type>::operator ()(complexe x){ 
  complexe res(0.0,0.0); 
  complexe prod(0.0,0.0);
  for(int i =0;i<deg+1;i++){ 
    prod =complexe(1.0,0.0); 
    for(int j=0;j<i;j++){ 
      prod = prod*x; 
    } 
    res = res + coefs[i]*prod; 
  } 
  return res;
}

template <double (*weight)(double), class segment_type> polynome<weight,segment_type> & polynome<weight,segment_type>::operator =(const polynome<weight,segment_type> & p){ 
  if(&p != this){ 
    deg = p.deg; 
    if(coefs != NULL){ 
      delete [] coefs;
    } coefs = NULL; 
    if(p.coefs != NULL){ 
      coefs = new complexe[deg+1]; 
      for(int i =0;i<deg+1;i++){ 
        coefs[i]=p.coefs[i]; 
      } 
    } 
  }
  return *this;
}

template <double (*weight)(double),class segment_type> polynome<weight,segment_type> polynome<weight,segment_type>:: operator * (polynome<weight,segment_type> p){ 
  int degt=deg+p.deg; 
  polynome<weight,segment_type> res(degt); 
  polynome<weight,segment_type> temp, temp2;
  temp = this->convert(degt); 
  temp2 = p.convert(degt); 
  for(int i=0;i<degt+1;i++){ 
    for(int j=0;j<i+1;j++){ 
      res.coefs[i]=res.coefs[i]+temp.coefs[j]*temp2.coefs[i-j]; 
    } 
  } 
return res;
} 


template <double (*weight)(double),class segment_type> polynome<weight,segment_type> polynome<weight,segment_type>::derivative(){ 
  polynome<weight,segment_type> res(deg-1); 
  for(int i =1;i<deg+1;i++){ 
    res.coefs[i-1]= i*coefs[i]; 
  } return res;
};
template <double (*weight)(double),class segment_type> polynome<weight,segment_type> polynome<weight,segment_type>::primitive(){ 
  polynome<weight,segment_type> res(deg+1); 
  for(int i=0;i<deg+1;i++){ 
    res.coefs[i+1]= (1.0/(i+1.0))*coefs[i]; 
  } 
  res.coefs[0]=complexe(0.0,0.0); 
  return res;
}
template <double (*weight)(double),class segment_type> complexe polynome<weight,segment_type>::integral(){ 
  complexe res(0.0,0.0); 
  complexe lenght(0.0,0.0);
  complexe ea, eb;
  lenght =complexe(segment_type::b,0.0)-complexe(segment_type::a,0.0);
  ea = complexe(weight(segment_type::a),0.0);
  eb = complexe(weight(segment_type::b),0.0);
  res=(*this)(complexe(segment_type::b,0.0))*eb +(*this)(complexe(segment_type::a,0.0))*ea; 
  return res;
}

template <double (*weight)(double),class segment_type> complexe polynome<weight,segment_type>:: operator,(polynome<weight,segment_type> p){ 
  polynome pro, prim; 
  complexe res(0.0,0.0);
  pro = (*this)*p.conjuge(); 
  res = pro.integral(); 
  return res;
}
template <double (*weight)(double),class segment_type> double polynome<weight,segment_type>::norm(){ 
  complexe n(0,0); 
  n = (*this,*this); 
  return sqrt(n.module());
};


#endif
