#ifndef anneauquad_HPP
#define anneauquad_HPP

#include <cmath>
#include <iostream>
#include "rationnel.hpp"

using namespace std;

template <class T, int d=1> class anneauquad{
  T a;
  T b;
public:
  anneauquad (){
    a = T();
    b = T();
  }
  anneauquad (T ainit , T binit){
    a = ainit; b = binit;
  }
  anneauquad (const anneauquad & x){
    a = x.a;  b = x.b;
  }
  ~anneauquad (){
  }
  anneauquad set_a (T);
  anneauquad set_b (T);
  anneauquad operator + (anneauquad);
  anneauquad operator - (anneauquad);
  anneauquad operator * (anneauquad);
  anneauquad & operator = (const anneauquad &);
  operator anneauquad<rationnel,2>();
  operator anneauquad<int,2>();
  
  template <class U, int e>
  friend double operator + (double, anneauquad<U,e>);
  template <class U, int e>
  friend double operator * (double, anneauquad<U,e>);
  template <class U, int e>
  friend ostream & operator << (ostream &, const anneauquad<U,e> &);
  int testu_z();
  int testu_q();
  int all_testsu();
};

template <class T, int d> anneauquad<T,d> anneauquad<T,d>::set_a (T aa){
    a = aa;
    return *this;
}

template <class T, int d> anneauquad<T,d> anneauquad<T,d>::set_b (T bb){
    b = bb;
    return *this;
}

template <>
anneauquad<int,0> anneauquad<int,0>::set_b (int bb){
  cout<<"Puisque d est nul la valeur de b est arbitraire"<<endl;
  return *this;
}


template <class T, int d> anneauquad<T,d> anneauquad<T,d>::operator + (anneauquad<T,d> x){
  anneauquad<T,d> res;
  res.a = a + x.a;
  res.b = b + x.b;
  return res;
}

template <class T, int d> anneauquad<T,d> anneauquad<T,d>::operator - (anneauquad<T,d> x){
  anneauquad<T,d> res;

  res.a = a + x.a;
  res.b = b + x.b;
  return res;
}

template <class T, int d> anneauquad<T,d> anneauquad<T,d>::operator * (anneauquad<T,d> x){
  anneauquad<T,d> res;

  res.a = a*x.a+d * b*x.b;
  res.b = a*x.b+b*x.a;
  return res;
}

template <class T, int d> anneauquad<T,d> & anneauquad<T,d>::operator = (const anneauquad<T,d> & x){
 if(&x != this){
    a = x.a;
    b = x.b;
  }
  return *this;
}

template <class T, int d> double operator * (double y, anneauquad<T,d> x){
  double res;

  res = y*( (double)x.a + sqrt(d) * (double)x.b);
  return res;
}

template <class T, int d> double operator + (double y, anneauquad<T,d> x){
  double res;

  res = y+( (double)x.a + sqrt(d) * (double)x.b);
  return res;
}

template <class T, int d> ostream & operator << (ostream & out, const anneauquad<T,d> & x){

  out <<x.a<<" + sqrt("<<d<<") "<<x.b<<endl;
  return out;
}

template<>
anneauquad<int,2>::operator anneauquad<rationnel,2>(){
  anneauquad<rationnel,2> x;
  x.set_a(rationnel(a,1));
  x.set_b(rationnel(b,1));
  return x;
}

template<>
anneauquad<long int,2>::operator anneauquad<rationnel,2>(){
  anneauquad<rationnel,2> x;
  x.set_a(rationnel(a,1));
  x.set_b(rationnel(b,1));
  return x;
}

template<>
anneauquad<rationnel,2>::operator anneauquad<int,2>(){
  anneauquad<int,2> x;
  x.set_a( (int) (a.num()/a.dem()));
  x.set_b( (int) (b.num()/b.dem()));
  return x;
}

  

template <class T, int d> int anneauquad<T,d>::testu_z(){
  anneauquad<int,2> x(1,1),y(1,2),z;
  double p= 2.5;
  *this = x; //(1 + sqrt(2))
  y = y + x;
  z = y * x; //8 + 5 sqrt(2)  
  p = (p + z) + p*x; //17.57106 + 6.03554 = 23.6066

  if(p > 23.6066 && p < 23.6067){ return 0; }
  else{ return 1; }
}


template <class T, int d> int anneauquad<T,d>::testu_q(){
  anneauquad<rationnel,3> x(rationnel(1,1),rationnel(1,2));
  anneauquad<rationnel,3> y(rationnel(1,1),rationnel(2,1));
  anneauquad<rationnel,3> z;
  double p= 1.2;
  
  *this = x; //((1/1) + (1/2)sqrt(3))
  y = y + x; // 2 +(5/2) sqrt(3))
  z = y * x; //23/4 + 7/2 sqrt(3)  = 11.812177
  p = (p + z) + p*x; // 13.012177 + 2.23923 = 15.251407

  if(p > 15.2514 && p < 15.2515){ return 0; }
  else{ return 1; }
}

template <class T, int d> int anneauquad<T,d>::all_testsu(){
  int c=0,t=0;
  anneauquad<int,2> a;
  c=a.testu_z(); t = t +c;
  anneauquad<rationnel,3> b;
  c=b.testu_q(); t = t +c;
  cout<<"Tests unitaires réussis, classe anneau: "<<2-t<<"/2"<<endl;
  return 0;
}



template <class T> class anneauquad2{
  T a;
  T b;
  int d;
public:
  anneauquad2 (){
    a = T();
    b = T();
    d =0;
  }
  anneauquad2 (T ainit , T binit, int dinit){
    a = ainit; b = binit; d = dinit;
  }
  anneauquad2 (const anneauquad2 & x){
    a = x.a;  b = x.b; d= x.d;
  }
  ~anneauquad2 (){
  }
  anneauquad2 set_a (T);
  anneauquad2 set_b (T);
  anneauquad2 operator + (anneauquad2);
  anneauquad2 operator - (anneauquad2);
  anneauquad2 operator * (anneauquad2);
  anneauquad2 & operator = (const anneauquad2 &);
  template <class U>
  friend double operator + (double, anneauquad2<U>);
  template <class U>
  friend double operator * (double, anneauquad2<U>);
  template <class U>
  friend ostream & operator << (ostream &, const anneauquad2<U> &);
};

template <class T> anneauquad2<T> anneauquad2<T>::set_a (T aa){
    a = aa;
    return *this;
}

template <class T> anneauquad2<T> anneauquad2<T>::set_b (T bb){
    b = bb;
    return *this;
}


template <class T> anneauquad2<T> anneauquad2<T>::operator + (anneauquad2<T> x){
  anneauquad2<T> res;
  if( d != x.d){
    cout<<" les nombres d ne sont pas egaux, l'addition interne n'est pas valide "<<endl;
  }
  res.a = a + x.a;
  res.b = b + x.b;
  return res;
}

template <class T> anneauquad2<T> anneauquad2<T>::operator - (anneauquad2<T> x){
  anneauquad2<T> res;
  if( d != x.d){
    cout<<" les nombres d ne sont pas egaux, l'addition interne n'est pas valide "<<endl;
  }
  res.a = a + x.a;
  res.b = b + x.b;
  return res;
}

template <class T> anneauquad2<T> anneauquad2<T>::operator * (anneauquad2<T> x){
  anneauquad2<T> res;
  if( d != x.d){
    cout<<" les nombres d ne sont pas egaux, l'addition interne n'est pas valide "<<endl;
  }
  
  res.a = a*x.a+d * b*x.b;
  res.b = a*x.b+b*x.a;
  return res;
}

template <class T> anneauquad2<T> & anneauquad2<T>::operator = (const anneauquad2<T> & x){
 if(&x != this){
    a = x.a;
    b = x.b;
  }
  return *this;
}

template <class T> double operator * (double y, anneauquad2<T> x){
  double res;

  res = y*( (double)x.a + sqrt(x.d) * (double)x.b);
  return res;
}

template <class T> double operator + (double y, anneauquad2<T> x){
  double res;

  res = y+( (double)x.a + sqrt(x.d) * (double)x.b);
  return res;
}

template <class T> ostream & operator << (ostream & out, const anneauquad2<T> & x){

  out <<x.a<<" + sqrt("<<x.d<<") "<<x.b<<endl;
  return out;
}

#endif
